function [R_final,R,time_sim,time_x0,time_steps,time_steps_sim,total_time,distance,vc,vd,vdout,tipo,other_solutions_p,x0]=compute_LIM_D(xe,par,mapcell,varargin)

tic
p=inputParser;

dim=length(xe); % dimension of the system (autonomous system)

% parameters for the simulation
DEFtfinal=100; % maximal final time of each simulation
DEFweight=ones(1,dim);
DEFnumber_of_steps=20; % number of simulation performed (this must be changed to an automatic method for finishing)
DEFnum_of_cell_around_equilibrium=3; % cells around the equilibrium to have faster convergence and avoid faulse new fixed points around it
DEFnum_fig=round(rand*10000); % number of the figure containint points
DEFplot_results=1; % 1: plot results after each iteration, 0: plot only at the end for faster computation
DEFplot_results_final=1; % 1: plot final results, 0: only provide numerical output
DEFspaceboundary=[xe-10, xe+10]; % boundaries of the phase space, example: spaceboundary=[x1min, x2min, x3min, x1max, x2max, x3max];
DEFdiscr=1001; % discretization number in each direction
DEFautomatic=0; % 0: in case of undefined time series it is asked to the user to decide about the convergence, 1: the algorithm proceed completely automatically imposing as diverging undefined time series
DEFvar1=1; % plotting variable 1 in the phase space
DEFvar2=2; % plotting variable 2 in the phase space
DEFrep_fix_point=100; % number of repetition in a cell before a new fixed point is defined
DEFrep_periodic=30; % number of repetition in a cell before a new periodic solution is defined
DEFinit_type = 3; % type of initial condition. 0: constant;  1: linear; 2: jump ; 3: free vibration;
DEFNtau = 10; % number of time intervalls within a time delay loop
DEFind_check = [1,3,6,9];     % indeces which are checked in the comparision of the previous diverging/converging trajectories
DEFind_check_conv = [1,2,3,4,5,6,7,8,9];   %ind check close to the equilibrium
DEFNbisec = 5; % number of steps with bisection method (for the initial cond)
DEFbisec_count_max = 3;    % the max number of bisec iterations after each other (to get random initial conditions as well)
DEFNrandom = 40;    % the maximum number of random initial conditions after each other. If it is reached a bisection method starts from the last initial condition

% optional input organization
addOptional(p,'tfinal',DEFtfinal);
addOptional(p,'weight',DEFweight);
addOptional(p,'number_of_steps',DEFnumber_of_steps);
addOptional(p,'num_of_cell_around_equilibrium',DEFnum_of_cell_around_equilibrium);
addOptional(p,'num_fig',DEFnum_fig);
addOptional(p,'plot_results',DEFplot_results);
addOptional(p,'plot_results_final',DEFplot_results_final);
addOptional(p,'spaceboundary',DEFspaceboundary);
addOptional(p,'discr',DEFdiscr);
addOptional(p,'automatic',DEFautomatic);
addOptional(p,'var1',DEFvar1);
addOptional(p,'var2',DEFvar2);
addOptional(p,'rep_fix_point',DEFrep_fix_point);
addOptional(p,'rep_periodic',DEFrep_periodic);
addOptional(p,'init_type',DEFinit_type);
addOptional(p,'Ntau',DEFNtau);
addOptional(p,'ind_check',DEFind_check);
addOptional(p,'ind_check_conv',DEFind_check_conv);
addOptional(p,'Nbisec',DEFNbisec);
addOptional(p,'bisec_count_max',DEFbisec_count_max);
addOptional(p,'Nrandom',DEFNrandom);

parse(p,varargin{:});
%% assign values
tfinal=p.Results.tfinal;
weight=p.Results.weight;
number_of_steps=p.Results.number_of_steps;
num_of_cell_around_equilibrium=p.Results.num_of_cell_around_equilibrium;
num_fig=p.Results.num_fig;
plot_results=p.Results.plot_results;
plot_results_final=p.Results.plot_results_final;
spaceboundary=p.Results.spaceboundary;
discr=p.Results.discr;
automatic=p.Results.automatic;
var1=p.Results.var1;
var2=p.Results.var2;
rep_fix_point=p.Results.rep_fix_point;
rep_periodic=p.Results.rep_periodic;
init_type=p.Results.init_type;
Ntau=p.Results.Ntau;
ind_check=p.Results.ind_check;
ind_check_conv=p.Results.ind_check_conv;
Nbisec=p.Results.Nbisec;
bisec_count_max=p.Results.bisec_count_max;
Nrandom = p.Results.Nrandom;

Ts=par.tau; % length of one short simulation equals to the time delay
num_of_timesteps = round(tfinal/Ts*Ntau*(number_of_steps+1));

other_solutions_p={};
 
time_steps=zeros(number_of_steps,1); % for measuring the time required for each step
time_sim=zeros(number_of_steps,1); % for measuring the time required for each simulation
time_x0=zeros(number_of_steps,1); % for measuring the time required for each search of initial consitions
distance=zeros(number_of_steps,1); % distance of each initial condition from other points
time_steps_sim=zeros(number_of_steps,1); % number of time steps of each simulation
x0=zeros(number_of_steps,dim); % vector containing all initial conditions utilized

%% this is for checking that dimension, spaceboundary and xe are inserted "legally"
checkoninput_D(dim,spaceboundary(1,:),xe,weight,[var1 var2])

%% inizialise converging and diverging points and cells
vc=zeros(num_of_timesteps,dim); % vector of converging trajectories (points)
vd=zeros(num_of_timesteps,dim); % vector of diverging trajectories (points)
vdout=zeros(num_of_timesteps,dim); % vector of diverging trajectories out of space boundary (points)
cell_c=zeros(num_of_timesteps,1); % vector of converging trajectories (cells)
cell_d=zeros(num_of_timesteps,1); % vector of diverging trajectories (cells)
cell_dout=zeros(num_of_timesteps,1); % vector of diverging trajectories out of space boundary (cells)
ind_cp=0; % index of converging points
ind_dp=0; % index of diverging points
ind_doutp=0; % index of diverging points out of the phase space
ind_dc=0; % index of diverging cells
ind_doutc=0; % index of diverging cells
counter_new_sol=0; % counter for new solutions
ibisec = 0;     % counter of bisection methon (initial cond)
irandom = 0;    % counter of random initial conditions 
bisec_count = 0; % counting the number of whole bisection iterations
randinit = 1;   % type of initial condition 1: random 0: bisection method, and based on the closest point.
found_div = 0;  % bisection method: whether a diverging trajectory is already found
tipo=zeros(number_of_steps,1); % type of convergence of each simulation
R=zeros(number_of_steps+1,1); % radius of convergence at each simulation step (for the next initial condition)
minimum=1e9; % large value for finding the initial R
for i=1:dim % look for the minimal distance between xe and phase space boundary
    minimum=min([minimum,weight(i)^0.5*abs(xe(i)-spaceboundary(i)),weight(i)^0.5*abs(xe(i)-spaceboundary(dim+i))]);
end
R(1)=minimum; % initial value of R

dh=zeros(dim,1); % step sizes in each dimension
for i=1:dim
    dh(i)=(spaceboundary(dim+i)-spaceboundary(i))/discr;
end

cell_pos=zeros(discr,dim); % vector including all the used coordinates in each direction (required for finding correct cell)
for i1=1:dim
    cell_pos(:,i1)=[spaceboundary(i1)+dh(i1)/2:dh(i1):spaceboundary(dim+i1)];
end

% find cell of equilibrium
cell_e=zeros(dim,1); % inizialize cell of equilibrium
for i=1:dim % for each coordinate, look for the corresponding cell index
    [~,cell_e(i)]=min(abs(cell_pos(:,i)-xe(i)));
end
%% updates initializing

cell_c(1:Ntau,1)=repmat(find_index_D(cell_e,dim,discr),Ntau,1); % assign to the converging cells the cell containing xe (here ind_cc=0)
ind_cc = Ntau;  % Ntau number of converging cells are already stored
vc(ind_cp+1,:)=xe; % assign xe to the converging points
ind_cp=ind_cp+1; % update counter of converging points
cell_f_around_eq=find_cells_around_a_cell_D(dim,cell_e,discr,num_of_cell_around_equilibrium); % find cells around equilibrium cell which are then assigned as converging
for i_eq = 1 : length(cell_f_around_eq) % asign to the converging cells the cells close to the equilibrium as well
    cell_c(ind_cc+1:ind_cc+Ntau+1,1) = [0;repmat(cell_f_around_eq(i_eq),Ntau,1)];
    ind_cc = ind_cc+Ntau+1;
end

x0(1,:)=randomIC_radius_unbalanced_D(dim,R(1),xe,weight);% first initial condition chosen with a random selection
%% cycle

for i1=1:number_of_steps % cycle over the pre-defined number of steps
    
    step_tic=tic; % initiate step timer
    sim_tic=tic; % initiate simulation timer
    % perform simulation, provides points, time interval of the point
    % series, cells touched, type of convergence, new solution in cells
    % (empty if the solution is not new), new solution in points (empty if
    % the solution is not new), number of points of the simulation
    [xt,t,cell_f_matr,tipo(i1),new_solution_c,new_solution_p,time_steps_sim(i1)]=simulation_D(dim,x0(i1,:),tfinal,Ts,Ntau,par.r,mapcell,spaceboundary,discr,cell_c(1:ind_cc,1),cell_d(1:ind_dc,1),cell_dout(1:ind_doutc,1),cell_pos,dh,weight,rep_fix_point,rep_periodic,ind_check,ind_check_conv,init_type);

    time_sim(i1)=toc(sim_tic); % record time of the simulation
    if tipo(i1)==0 % in the case of not converged time series, it is asked to the user to choose if he thinks that the time series converges to the desired solution or not, based on a displaced phase space figure
        if automatic == 0 % if the computation is not automatic (automatic==0), the user can deside if an undefined time series is converging or not
            figure;
            subplot(211);plot(t,xt(:,var1), t, xt(:,var2)); legend('q1','q2');
            subplot(212);plot(xt(:,var1),xt(:,var2),xe(var1),xe(var2),'.','MarkerSize',15);drawnow
            str=join(['The time series did not converge in the given time to any solution (known or unknown), this is the simulation number',string(i1)]);
            warning(str)
            while (tipo(i1)~=1 && tipo(i1)~=2)
                inputvalue = input('Based on the displayed figure, please, type 1 if you think that the solution is converging to the desired equilibrium, otherwise type 2 ');
                if ~isempty(inputvalue)
                    tipo(i1)= inputvalue;
                    if (tipo(i1)~=1 && tipo(i1)~=2)
                        disp('Not valid entry, only "1" and "2" are valid entries')
                    end
                end
            end
            close;
        else
            tipo(i1)=2; % if automatic, select as diverged
        end
    end
    
    switch tipo(i1) % depending on the type of convergence, different action are performed
        case 1 % converging to equilibrium
            vc(ind_cp+1:ind_cp+length(t),:)=xt; % save points in vc (vector of points of converging trajectories)
            if plot_results>0.5 % if chosen to plot at every step
                figure(num_fig);plot(vc(ind_cp+1:ind_cp+length(t),var1),vc(ind_cp+1:ind_cp+length(t),var2),'b.');hold on;drawnow % add the time series in the modal phase space (in 2D, by default q1 and q2)
            end
            ind_cp=ind_cp+length(t); % update index of converging points

            cell_f_matr_reshape = reshape(cell_f_matr(2:end-1,1:end-1).',[],1);
            cell_c(ind_cc+1:ind_cc+length(cell_f_matr_reshape)+1,1) = [0;cell_f_matr_reshape]; % update the vector of converging cells - the zero element separates the different trajectories
            ind_cc=ind_cc+length(cell_f_matr_reshape)+1; % update index of converging cells
            
            R(i1+1)=R(i1); % value of radius of convergence does not change
        case 2 % converging to an already known undesired solution
            vd(ind_dp+1:ind_dp+length(t),:)=xt; % save points in vd (vector of points of diverging trajectories)
            if plot_results>0.5 % if chosen to plot at every step
                figure(num_fig);plot(vd(ind_dp+1:ind_dp+length(t),var1),vd(ind_dp+1:ind_dp+length(t),var2),'r.');hold on;drawnow % add the time series in the modal phase space (in 2D, by default q1 and q2)
            end
            ind_dp=ind_dp+length(t);  % update index of diverging points
            
            cell_f_matr_reshape = reshape(cell_f_matr(2:end-1,1:end-1).',[],1);
            cell_d(ind_dc+1:ind_dc+length(cell_f_matr_reshape)+1,1)=[0;cell_f_matr_reshape]; % update the vector of diverging cells - the zero element separates the different trajectories
            ind_dc=ind_dc+length(cell_f_matr_reshape)+1; % update index of diverging cells
            
            R(i1+1)=min(R(i1),findradius_D(x0(i1,:),xe,weight)); % calculate new radius of convergence, comparing the old one with the one for the new points
        case 3 % diverging out of the phase space
            vdout(ind_doutp+1:ind_doutp+length(t),:)=xt; % save points in vdout (vector of diverging points out of the phase space)
            if plot_results>0.5 % if chosen to plot at every step
                figure(num_fig);plot(vdout(ind_doutp+1:ind_doutp+length(t)-1,var1),vdout(ind_doutp+1:ind_doutp+length(t)-1,var2),'k.');hold on;drawnow % add the time series in the modal phase space (in 2D, by default q1 and q2)
            end
            ind_doutp=ind_doutp+length(t);  % update index of diverging outside points
            
            cell_f_matr_reshape = reshape(cell_f_matr(2:end-1,1:end-1).',[],1);
            cell_dout(ind_doutc+1:ind_doutc+length(cell_f_matr_reshape)+1,1) = [0;cell_f_matr_reshape]; % update the vector of diverging outside cells - the zero element separates the different trajectories
            ind_doutc=ind_doutc+length(cell_f_matr_reshape)+1;
            
            R(i1+1)=min(R(i1),findradius_D(x0(i1,:),xe,weight)); % calculate new radius of convergence, comparing the old one with the one for the new points
        case 4 % converging to a new unknown solution
            vd(ind_dp+1:ind_dp+length(t),:)=xt; % save points in vd (vector of diverging points)
            if plot_results>0.5 % if chosen to plot at every step
                figure(num_fig);plot(vd(ind_dp+1:ind_dp+length(t),var1),vd(ind_dp+1:ind_dp+length(t),var2),'r.');hold on;drawnow % add the time series in the modal phase space (in 2D, by default q1 and q2)
            end
            ind_dp=ind_dp+length(t);  % update index of diverging points
            
            cell_f_matr_reshape = reshape(cell_f_matr(2:end-1,1:end-1).',[],1);
            cell_d(ind_dc+1:ind_dc+length(cell_f_matr_reshape)+1,1) = [0;cell_f_matr_reshape]; % update the vector of diverging cells - the zero element separates the different trajectories
            ind_dc=ind_dc+length(cell_f_matr_reshape)+1;
            
            R(i1+1)=min(R(i1),findradius_D(x0(i1,:),xe,weight)); % calculate new radius of convergence, comparing the old one with the one for the new points
            
            counter_new_sol=counter_new_sol+1; % update counter of new solution
            other_solutions_c(counter_new_sol,:)={new_solution_c}; % add cells of the solution to the collector
            other_solutions_p(counter_new_sol,:)={new_solution_p}; % add points of the solution to the collector of the solution
            if plot_results>0.5 % if chosen to plot at every step
                if size(new_solution_p,1)==1
                    figure(num_fig);plot(other_solutions_p{counter_new_sol}(:,var1),other_solutions_p{counter_new_sol}(:,var2),'g.','MarkerSize',30);hold on;drawnow % plot the new obtained solution
                else
                    figure(num_fig);plot(other_solutions_p{counter_new_sol}(:,var1),other_solutions_p{counter_new_sol}(:,var2),'k--','LineWidth',2);hold on;drawnow % plot the new obtained solution
                end
            end
    end
    
    if i1<number_of_steps % skip this passage for the last computation
        x0_tic=tic; % initialize counter of search for initial condition
        % look for the next initial condition as the most remote point
        % (approximate) within the convergence circle. It also provide the
        % distance from the new staring point and the closest point inside
        % the convergence circle
        if i1 == 1
            r_bisec = R(i1+1); % initializing of the closest distance
        end
        
        if tipo(i1) == 1 && randinit == 1 && irandom < Nrandom % random initial condition untill a diverging trajectory is found or until the maximum number of random initial conditions
            x0(i1+1,:)=randomIC_radius_unbalanced_D(dim,R(i1+1),xe,weight);
            irandom = irandom +1; % increase the counter of subsequent random initial conditions
        else  %bisection method
            randinit = 0;
            ibisec = ibisec+1;      % update the counter of iterations during the bisection method
            if tipo(i1)~=1  % divergind trajectory
                if findradius_D(xt,xe,weight) < r_bisec % store the closest point of the previous diverging trajectories
                    [r_bisec,id_bisec] = findradius_id_D(xt,xe,weight);
                    xclosest = xt(id_bisec,:);
                end
                found_div = 1;  % at least one diverging trajectory is found during the bisection method                   
            end
            
            if ibisec < Nbisec
                if found_div == 0   % until now in this round only stable trajectories were found -> the next initial condition should be farther away from the equilibrium
                    if findradius_D(x0,xe,weight) < R(i1+1)*0.8   % if the actual distance from the equilibrium is smaller than 80% of the current LIM the next init. cond. will be twice as far from the origin
                        bisecmin = x0(i1,:);
                        bisecmax = x0(i1,:)+x0(i1,:)-xe;
                    else % if the actual distance from the equilibrium is larger than 80% of the current LIM the next init. cond. will be 1.25 times farther 
                        bisecmin = x0(i1,:);
                        bisecmax = x0(i1,:)+(x0(i1,:)-xe)*0.25;
                    end
                        
                    for i=1:dim % if the maximum distance is outside the space boundary then it projected to the space boundary
                       if bisecmax(i)>spaceboundary(dim+i)
                           bisecmax = xe+(bisecmax-xe)*(spaceboundary(dim+i)-xe(i))/(bisecmax(i)-xe(i));
                       end
                       if bisecmax(i)<spaceboundary(i)
                           bisecmax = xe+(bisecmax-xe)*(spaceboundary(dim+i)-xe(i))/(bisecmax(i)-xe(i));
                       end                   
                    end
                    if max(max(x0==bisecmax)) %the chosen point was already checked -> the next initial condition is chosen randomly
                        randinit = 1;   % random initial condition will be used
                        x0(i1+1,:)=randomIC_radius_unbalanced_D(dim,R(i1+1),xe,weight);   % random initial condition
                        bisec_count = 0; % initialize for the next bisection iteration
                        ibisec = 0; % initialize for the next bisection iteration
                        irandom = 0; % initialize for the next bisection iteration                
                    else
                        x0(i1+1,:) =  bisecmax;     % new initial condition
                    end
                else            % if in this round at least 1 div is found
                    if ibisec == 1  % if the first one is a diverging trajectory
                        bisecmin = xe;  
                        bisecmax = x0(i1,:);
                    elseif tipo(i1) == 1    % in case of a converging trajectory the min limit is updated (increased)
                        bisecmin = x0(i1,:);
                    else
                        bisecmax = x0(i1,:); % in case of a diverging trajectory the max limit is updated (decreased)
                    end
                    x0(i1+1,:) =  (bisecmin+bisecmax)/2;    % new initial condition
                end
            elseif ibisec == Nbisec     % last step within the current bisection method
                if r_bisec<R(i1+1) && bisec_count < bisec_count_max  && ~max(max(x0==xclosest))   % if the number of whole bisection iterations is still smaller than the allowed number and a divering trajectory intersected the current hypersphere of convergence
                    x0(i1+1,:) = xclosest;      % the next initial point is that point of the diverging trajectories which was closest to the desired equilibrium
                    found_div = 0;  % initialize for the next bisection iteration
                    ibisec = 0;  % initialize for the next bisection iteration
                    bisec_count = bisec_count+1;    % update the counter of the whole bisection iterations
                else
                    randinit = 1;   % random initial condition will be used
                    x0(i1+1,:)=randomIC_radius_unbalanced_D(dim,R(i1+1),xe,weight);   % random initial condition
                    bisec_count = 0; % initialize for the next bisection iteration
                    ibisec = 0; % initialize for the next bisection iteration
                    found_div = 0; % initialize for the next bisection iteration
                    irandom = 0; % initialize for the next bisection iteration
                end
            end
        end
        time_x0(i1)=toc(x0_tic); % measure time for choosing new initial condition
        if plot_results>0.5 % if chosen to plot at every step
            figure(num_fig);hold on;plot(x0(i1+1,var1),x0(i1+1,var2),'xk','LineWidth',2,'MarkerSize',10);drawnow % mark the new initial condition on the figure
        end
    end
    time_steps(i1)=toc(step_tic); % measure the time for the step just concluded
end
vdout=vdout(1:ind_doutp,:); % eliminate zeros from the vector of diverging outside points
vd=vd(1:ind_dp,:); % eliminate zeros from the vector of diverging points
vc=vc(1:ind_cp,:); % eliminate zeros from the vector of converging points
if plot_results_final>0.5 % check if it is required to plot final figures or not
    figure;plot(R);xlabel('iteration');ylabel('LIM'); % plot the radius of convergence trend
    figure;plot(time_steps);hold on;plot(time_sim);plot(time_x0);xlabel('iterations');ylabel('time utilized'); % plot the times taken for the iterations
end
R_final=R(end);
total_time=toc; % save total time
if plot_results>0.5
    figure(num_fig)
    title('modal coordinates')
    xlabel('var1')
    ylabel('var2')
end
